package bookExamples.ch26Graphics.draw2d;


import java.awt.*;

public class Circle2d extends DJShape
        implements Intersects, Movable {
    private int diameter = 1;
    private int radius = 1;
     Vec2d center = new Vec2d(0, 0);
    private double radiusSq;

    public Circle2d(int x1, int y1, int diameter) {
        super(x1, y1, x1 + diameter, y1 + diameter);
        this.x1 = x1;
        this.y1 = y1;
        this.diameter = diameter;
        radius = this.diameter / 2;
        radiusSq = radius * radius;
        setCenter();

    }

    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        Circle2d circle2d = new Circle2d(x1, y1, x2, y2);
        setForeground(Color.MAGENTA);
        return circle2d;
    }


    public Circle2d(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        x1 = _x1;
        y1 = _y1;
        int dx = x1 - _x2;
        int dy = y1 - _y2;
        diameter = (int) Math.sqrt(dx * dx + dy * dy);
        if (_x1 > _x2) x1 = _x2;
        if (_y1 > _y2) y1 = _y2;
        radius = diameter / 2;
        radiusSq = radius * radius;
        setCenter();
    }

    private void setCenter() {
        center.v[0] = x1 + radius;
        center.v[1] = y1 + radius;
    }

    public Point getCenter() {
        return new Point((int) center.v[0], (int) center.v[1]);
    }

    public void draw(Graphics g) {
        //saveState(g);

        Point pc = getCenter();

        Graphics2D g2d = (Graphics2D) g;
        g2d.setColor(getForeground());

        g2d.drawOval(x1, y1, diameter, diameter);

        g2d.fillOval(pc.x, pc.y, 2, 2);
        g2d.drawString("(" + pc.x + "," + pc.y + ")", pc.x + 3, pc.y + 3);

        //Point p = new Point(xc-radius,yc);
        //Point pdest = new Point();
        //at.transform(p,pdest);
        g2d.drawLine(pc.x, pc.y, pc.x - radius, pc.y);
        //g.drawLine(xc, yc, (int) pdest.getX(), (int) pdest.getY());
        //restoreState(g2d);
    }

    public String toString() {
        Point pc = getCenter();
        return
                "xc,yc,r=" + pc.x + "," + pc.y + "," + radius;
    }


    public boolean inside(Vec2d p) {

        Vec2d n = new Vec2d(p);
        n.sub(center);
        if (n.dot(n) > radiusSq)
            return false;
        else
            return true;
    }

    public Vec2d intersect(Ray2d ray) {

        double t, /* where the ray intersects */
                // vDotV = distance square from sphere center to
                // ray origin
                vDotV,
                dDotV,
                thc;        /* length sqare of the half chord */

        // vector to the center of the sphere from
        // ray origin
        Vec2d v  = new Vec2d(center.v[0], center.v[1]);
        boolean inside = false;
        //System.out.println(ray);

        /* use the closest approach algorithm */
        // v = v - ray.p
        //System.out.println(v);
        v.sub(ray.p);
        // loc = v^2
        vDotV = v.dot(v);
        // Are we inside the circle?
        if (vDotV <= radiusSq)
            inside = true;

        // dDotV = D dot V =
        // cos(angle between ray and vec to center);
        dDotV = v.dot(ray.d);   /* find the closest approach */

        // object is behind the ray origin
        if ((inside != true) && (dDotV <= 0.0))
            return null;

        /* compute the half chord square from the ray intersection to the
           intersection normal. */
        thc = (dDotV * dDotV) - vDotV + radiusSq;
        if (thc < 0.0)
            return null;   /* ray misses the sphere */

        /* find the ray intersection */
        if (inside == true)
            t = dDotV + Math.sqrt(thc);
        else
            t = dDotV - Math.sqrt(thc);

        return ray.vecOnLine((double) t);
    }

    public void move(double x, double y) {
        center = new Vec2d(x, y);
    }
}
